#ifndef ITERABLENEIGHBORS_HH_
#define ITERABLENEIGHBORS_HH_

#include "ell/State.hh"

namespace ell
{

	 /*!
	  * The IterableNeighbors interface allows a generic NeighborList for 
	  * neighborhood generation. Therefore, a neighbor indexing and access has 
	  * to be provided by the implementing State subclass.
	  * 
	  * @author Martin Mann
	  */
	class IterableNeighbors
	{
	public:
		
		virtual ~IterableNeighbors() {};

		  //! How many neighbor indices are accessible.
		  //! @return the number of neighbors
		virtual size_t getNeighborNumber(void) const = 0;
		
		  //! Access to a specific neighbor.
		  //! @param index the index of the neighbor in [0, getNeighborNumber())
		  //! @param neigh if != NULL this state should be converted into the 
		  //!              neighbor otherwise a new one is created and returned
		  //! @return the new neighbor state or NULL if the index is >= 
		  //! getNeighborNumber()
		virtual State* getNeighbor(const size_t index, State* neigh) const = 0;
		
		  //! Undo the changes done to this state to generate the given neighbor.
		  //! The goal is to do less operations than doing a full copy like
		  //! (*neigh = *this).
		  //! @param index the neighbor index that was applied last 
		  //!              [ 0, getNeighborNumber() )
		  //! @param neigh the state resulting from the last changes
		  //! @return the copy of this object via undoing the last changes in 
		  //!         neigh or NULL if neigh == NULL
		virtual State* undoNeighborChange(const size_t index, State* const neigh) const = 0;
		
		  //! Applies the necessary changes to a given state to get the neighbor.
		  //! Note: \@copy is expected to be a copy (i.e. *this == *copy) and
		  //! therefore direct changes are possible and != NULL. 
		  //! @param index the neighbor to generate
		  //! @param copy a copy of *this the changes should be applied to
		  //! @return copy = the neighbor state of the given index or NULL if the index
		  //!         is out of range
		virtual State* applyNeighborChange(const size_t index, State* const copy) const = 0;
		
	}; // class
	
	
	 /*!
	  * A generic NeighborList based on the IterableNeighbors interface that
	  * successively generates all neighbors of a given state. 
	  * 
	  * @author Martin Mann
	  */
	class SuccessiveNeighborList : public State::NeighborList {
	
	protected:
		const IterableNeighbors *source;
		
		//! encodes the last neighbor index used
		class ItState : public State::NeighborList::ItState {
		public:
			size_t idx; //!< position to flip
			ItState():idx(0) {}
			ItState(const ItState &itstate) : idx(itstate.idx) {}
			virtual ItState * clone() const { return new ItState(*this); }
		};
    
	public:
		SuccessiveNeighborList(const IterableNeighbors* source_);
    
		virtual ~SuccessiveNeighborList();
  
		/*! Returns a pointer to the next element of the
		 *  virtual list of neighbors of the state of origin.
		 *  Returns NULL if no more neighbors exist.
		 * @param itstate constains state of the list iterator
		 * @param elem this State is used for the next neighbor
		 * @return next element in NeighborList 
		 */
		virtual State* 
		next(State::NeighborList::ItState *itstate, State* const elem) const;

		/*! Returns a pointer to the first element of the
		 *  virtual list of neighbors of the state of origin.
		 *  Returns NULL if no neighbors exist.
		 * @param itstate constains state of the list iterator
		 * @return first element of NeighborList
		 */
		virtual State* first(State::NeighborList::ItState** itstate) const;
	}; // class 

	 /*!
	  * A generic NeighborList based on the IterableNeighbors interface that
	  * randomly generates all neighbors of a given state. 
	  * 
	  * @author Martin Mann
	  */
	class RandomNeighborList : public State::NeighborList {
	
	protected:
		const IterableNeighbors *source;
		std::vector<size_t> idx2apply;
		
		//! encodes the last neighbor index used
		class ItState : public State::NeighborList::ItState {
		public:
			size_t pos; //!< position to flip
			ItState():pos(0) {}
			ItState(const ItState &itstate) : pos(itstate.pos) {}
			virtual ItState * clone() const { return new ItState(*this); }
		};
    
	public:
		RandomNeighborList(const IterableNeighbors* source);
    
		virtual ~RandomNeighborList();
  
		/*! Returns a pointer to the next element of the
		 *  virtual list of neighbors of the state of origin.
		 *  Returns NULL if no more neighbors exist.
		 * @param itstate constains state of the list iterator
		 * @param elem this State is used for the next neighbor
		 * @return next element in NeighborList 
		 */		
		virtual State* 
		next(State::NeighborList::ItState *itstate, State* elem) const;

		/*! Returns a pointer to the first element of the
		 *  virtual list of neighbors of the state of origin.
		 *  Returns NULL if no neighbors exist.
		 * @param itstate constains state of the list iterator
		 * @return first element of NeighborList
		 */
		virtual State* first(State::NeighborList::ItState** itstate) const;
	}; // class 

} // namespace ell

#endif /*ITERABLENEIGHBORS_HH_*/
