#include "ell/NeighborGenerator.hh"
#include "ell/State.hh"
#include <iostream>
#include "biu/RandomNumberFactory.hh"

using namespace ell;

/*!
 * This is a dummy implementation of a State class including inner classes
 * NeighborList and ItState. It has a limited implementation of the functions
 * getRandomNeighbor, getNeighborList and getRandomNeighborList.
 * 
 * @author Daniel Maticzka
 */
class DummyState : public State
{
public:
	class NeighborList : public State::NeighborList 
	{
	private:
		std::string id;
		double origin;
	public:
		class ItState : public State::NeighborList::ItState {
		public:
			int pos;
			
			ItState():pos(0) {}
			
			ItState(const ItState &itstate) {pos=itstate.pos;}
			
			virtual ItState * clone() const {
				return new ItState(*this);
			}
		};
    
		NeighborList(std::string _id, double _origin) 
		{
			id = _id;
			// first successor starts with relative energy +3
			origin = _origin + 4;
		}
    
		virtual ~NeighborList() {}
  
		virtual State*
		next(State::NeighborList::ItState *itstate, State* elem) const {
			((DummyState*)elem)->applyMove();
			return elem;
		}

		virtual State* 
		first(State::NeighborList::ItState** itstate) const {
			DummyState* s = new DummyState(id, origin);
			s->applyMove();
			return s;
		}
	};
	
	std::string id;
	double energy;
	double maxEnergy;

	DummyState(std::string _id, double _energy) 
		{ id = _id; energy = _energy;} 
	~DummyState() {}
	
	bool operator== (const State& state2) const { return true; }
	bool operator!= (const State& state2) const { return true; }

	int	getFitness() const { return 0; }
	double getEnergy() const { return energy; }
	unsigned int getMinimalDistance(const State& state2) const { return 0; }
	State* clone() const { return new DummyState(id, energy); }
	State* fromString(const std::string& stringRep) const { return NULL; }
	std::string	toString() const { return id; }
	
	NeighborListPtr getNeighborList() const 
	{ 
		return NeighborListPtr(new NeighborList(id,energy));
	}
	
	NeighborListPtr getRandomNeighborList() const {
		return NeighborListPtr(new NeighborList(id,energy+5));
	}
	
	State* getRandomNeighbor(State* inPlaceNeigh = NULL) const 
	{ 
		int randomModifier = (int) ((((double) biu::RNF::getRN()) / ((double) biu::RNF::getMaxRN()))*10)-5;
		return new DummyState(id,energy + randomModifier); 
	}
	
	DummyState& applyMove() {
		// decrease energy by one
		energy -= 1;
		return *this;
	}

	//////////////  COMPRESSED REPRESENTATION  ////////////////////////////
	
	CSequence  compress(void) const {
		return CSequence();
	}
	
	CSequence&   compress(CSequence& toFill) const {
		toFill.clear();
		return toFill;
	}
	
	State*   uncompress(const CSequence& cseq, State* toFill) const {
		return toFill;
	}
	
	State*   uncompress(const CSequence& cseq) {
		return NULL;
	}

};

int main(int argc, char** argv)
{
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : NeighborGenerator\n"
				<<"===========================================" <<std::endl;
	
	 // create RNG and set for global use via biu::RNF 				
	biu::RandomNumberGenerator* rng = new biu::RNG_LCG();
	biu::RNF::setRNG(rng);	// set for global usage
	delete rng;

	std::cout 	<< "=> creating DummyState" << std::endl;
	double energy = 20;
	State* s = new DummyState("dummy",energy);
	
	std::cout 	<< "=> creating NG_Random" << std::endl;
	NeighborGenerator* ng0 = new NG_Random();
	std::cout 	<< "->getNeighborList(s)" << std::endl;
	State::NeighborListPtr nl = ng0->getNeighborList(s);
	std::cout	<< "=> accessing 10 Neighbors" << std::endl;
	State::NeighborList::Iterator it = nl->begin();
	for (int i=0; i<10; i++) {
		std::cout << "->getEnergy() = " << (*it).getEnergy() << std::endl;
		++it;
	}
	delete ng0;
	
	std::cout 	<< "=> creating NG_SuccessiveRandom" << std::endl;
	NeighborGenerator* ng1 = new NG_SuccessiveRandom();
	std::cout 	<< "->getNeighborList(s)" << std::endl;
	nl = ng1->getNeighborList(s);
	std::cout	<< "=> accessing 10 Neighbors" << std::endl;
	it = nl->begin();
	for (int i=0; i<10; i++) {
		std::cout << "->getEnergy() = " << (*it).getEnergy() << std::endl;
		++it;
	}
	delete ng1;

	std::cout 	<< "=> creating NG_Successive" << std::endl;
	NeighborGenerator* ng2 = new NG_Successive();
	std::cout 	<< "->getNeighborList(s)" << std::endl;
	nl = ng2->getNeighborList(s);
	std::cout	<< "=> accessing 10 Neighbors" << std::endl;
	it = nl->begin();
	for (int i=0; i<10; i++) {
		std::cout << "->getEnergy() = " << (*it).getEnergy() << std::endl;
		++it;
	}
	delete ng2;
	
	delete s;

	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	return 0;
}
