#include "ell/Walk.hh"
#include "ell/NeighborGenerator.hh"
#include "ell/StateAcceptor.hh"
#include "ell/WalkAbortionCriterion.hh"
#include "ell/SC_Listing.hh"
#include "ell/State.hh"
#include <iostream>
#include <string>
#include <vector>

using namespace ell;

/*!
 * This is a dummy implementation of a State class including inner classes
 * NeighborList and ItState.
 * 
 * @author Daniel Maticzka
 */
class DummyState : public State
{
public:
	class NeighborList : public State::NeighborList 
	{
	private:
		std::string id;
		double energy;
	public:
		class ItState : public State::NeighborList::ItState {
		public:
			int pos;
			
			ItState():pos(0) {}
			
			ItState(const ItState &itstate) {pos=itstate.pos;}
			
			virtual ItState * clone() const {
				return new ItState(*this);
			}
		};
    
		NeighborList(std::string _id, double _energy) 
		{
			id = _id;
			energy = _energy;
		}
    
		virtual ~NeighborList() {}
  
		virtual State*
		next(State::NeighborList::ItState *itstate, State* elem) const {
			((DummyState*)elem)->applyMove();
			return elem;
		}

		virtual State* 
		first(State::NeighborList::ItState** itstate) const {
			DummyState* s = new DummyState(id, energy);
			s->applyMove();
			return s;
		}
	};
	
	std::string id;
	double energy;

	DummyState(std::string _id, double _energy) { id = _id; energy = _energy; } 
	~DummyState() {}
	
	bool operator== (const State& state2) const { return true; }
	bool operator!= (const State& state2) const { return true; }

	int	getFitness() const { return 0; }
	double getEnergy() const { return energy; }
	unsigned int getMinimalDistance(const State& state2) const { return 0; }
	State* clone() const { return new DummyState(id, energy); }
	State* fromString(const std::string& stringRep) const { return NULL; }
	std::string	toString() const { return id; }
	NeighborListPtr getNeighborList() const 
	{ 
		return NeighborListPtr(new NeighborList(id,energy));
	}
	NeighborListPtr getRandomNeighborList() const {
		return NeighborListPtr(new NeighborList(id,energy));
	}
	State* getRandomNeighbor(State* inPlaceNeigh = NULL) const { return NULL; }
	DummyState& applyMove() {
		energy -= 1;
		return *this;
	}
	//////////////  COMPRESSED REPRESENTATION  ////////////////////////////
	
	CSequence  compress(void) const {
		return CSequence();
	}
	
	CSequence&   compress(CSequence& toFill) const {
		toFill.clear();
		return toFill;
	}
	
	State*   uncompress(const CSequence& cseq, State* toFill) const {
		return toFill;
	}
	
	State*   uncompress(const CSequence& cseq) {
		return NULL;
	}

};

int main(int argc, char **argv) {
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : Walk\n"
				<<"===========================================" <<std::endl;

	std::cout	<< "=> creating DummyState" << std::endl;
	State* state = new DummyState("state 1", 10);
	std::cout	<< "->getEnergy = " << state->getEnergy() << std::endl;
	
	std::cout	<< "=> creating NeighborGenerator" << std::endl;
	NeighborGenerator* ng = new NG_Successive();
	std::cout	<< "=> creating StateAcceptor" << std::endl;
	StateAcceptor* sa = new SA_E_Le();
	std::cout	<< "=> creating WalkAbortionCriterion" << std::endl;
	WalkAbortionCriterion* wac = new WAC_MaxLength(5);
	std::cout	<< "=> creating StateCollector scAccepted" << std::endl;
	StateCollector* scAccepted = new SC_Listing();
	std::cout	<< "=> creating StateCollector scChecked" << std::endl;
	StateCollector* scChecked = new SC_Listing();
	scChecked->add(*state);
	scChecked->add(*state);
	scChecked->add(*state);
				
	std::cout	<< "=> calling walk" << std::endl;
	scAccepted = Walk::walk(state, scAccepted, ng, sa, wac, scChecked);
	
	std::cout 	<< "Number Of Accepted States: " 
					<< scAccepted->size()
					<< std::endl;
	
	SC_Listing* sch = dynamic_cast<SC_Listing*>(scAccepted);
	
	std::cout 	<< "Accepted States:" << std::endl;
	SC_Listing::List list = sch->getList();
	SC_Listing::List::iterator iter;
  	for( iter = list.begin(); iter != list.end(); iter++ ) {
    	std::cout 	<< "  * Energy = "
    				<< (*iter)->getEnergy() << std::endl;
  	}
  	
	sch = dynamic_cast<SC_Listing*>(scChecked);
	
	std::cout 	<< "Checked States:" << std::endl;
	list = sch->getList();
  	for( iter = list.begin(); iter != list.end(); iter++ ) {
    	std::cout 	<< "  * Energy = "
    				<< (*iter)->getEnergy() << std::endl;
  	}
	
  	delete state;
  	delete ng;
  	delete sa;
  	delete wac;
  	delete scAccepted;
  	delete scChecked;
  	
  	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
  	
	return 0; 
}
